;; Generated package description from minuet.el  -*- mode: lisp-data; no-byte-compile: t; lexical-binding:t -*-
(define-package "minuet" "0.10.0.0.20260825.1" "Code completion using LLM" '((emacs "29") (plz "0.9") (dash "2.19.1")) :commit "2614e70f5e3f52a621c8ea12bacb24ebd6a69e13" :authors '(("Milan Glacier" . "dev@milanglacier.com")) :maintainer '("Milan Glacier" . "dev@milanglacier.com") :url "https://github.com/milanglacier/minuet-ai.el")
